/*********************************************************************/
/* File: clientserver.h                             Date 4/7 -98     */
/* Based on popproxy version 1.3 by N.Oren                           */
/* Programmer: Jens Landgr                                          */
/*********************************************************************/

#ifndef CLIENTSERVER_DEF
#define CLIENTSERVER_DEF

#include "buffer.h"
#include "boolean.h"

#define MAXCOMLEN 4
#define BUFSIZE 1000
#define IDLEN 10

enum POP_Command {
  C_OK, C_ERR, C_END, C_AUTH, C_USER, C_PASS, C_TOP, 
  C_LIST, C_RETR, C_UIDL, C_QUIT, C_NONE
};

/********************************************************************/
/* Declaration of class popstreamClass                              */
/********************************************************************/
class popstreamClass
{
private:
  char _buf[BUFSIZE];
  /* Is set on a error, avoids infinite loops, since panic() tries
     to write error signals. */
  int _malfunction;
  bufferClass *_sock;
  char _id[IDLEN];

  void init();

public:
  popstreamClass(bufferClass *sock);

  popstreamClass(bufferClass *sock, char *id);

  ~popstreamClass();

  /* Reads from mailserver */
  char *read();

  /* Writes to mailserver */
  char *write(char *buf);

  /* Gets the command of the current data. */
  POP_Command getCom();

  /* Checks if the current command is c */
  boolean checkCom(POP_Command c);

  /* Returns the current data. */
  char *getData();

  /* Sends a command, with data */
  void sendCom(POP_Command com, char *data);

  /* Sends a command, without data */
  void sendCom(POP_Command com);

  /* Transfers a multirow responce from a popstream */
  void transfer(popstreamClass &ps);

  /* Returns username (only to be used when the USER command 
     have been recieved) */
  char *extractUname();

  /* Returns servername (same as above) */
  char *extractServname();
}; //class popstreamClass

#endif













