/******************************************************************/
/* File: Popenrw                                 Date: 31/7 -98   */
/* popen but both stdin and stdout is redirected.                 */
/* This one uses new and delete instead of malloc and free.       */
/******************************************************************/

/******************************************************************/
/* INCLUDES AND EXTERNS                                           */
/******************************************************************/
#include "popenrw.h"

#include <unistd.h>
#include <sys/wait.h>
#include <libio.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>

/******************************************************************/
/* DEFINES                                                        */
/******************************************************************/
#define STD_FD 0
#define ABNORMAL_RETURN -1
#define NORMAL_RETURN 1
#define EXIT_STATUS 127
#define CONNECT_REPETITIONS 5
#define FNAME_LEN 14 /* Including null */

/******************************************************************/
/* STRUCTS AND TYPEDEFS                                           */
/******************************************************************/
/* A linkable file with pid */
struct proc_file
{
  int fd;
  int pid;
  struct proc_file *next;
};

typedef struct proc_file proc_file;

static struct proc_file *proc_file_chain = NULL;

/******************************************************************/
/* FUNCTION DECLARATIONS                                          */
/******************************************************************/

void destroySock(int sock)
{
  shutdown(sock, 2); /* Is both these needed? */
  close(sock);
}

int proc_open(proc_file *fpx, const char *command)
{
  int sock[2];
  int child_pid;

  /* Should I give the sockets names?
     What does getpeername(sock[1]) return? */
  if (socketpair(AF_UNIX, SOCK_STREAM, 0, sock) == -1) {
    perror("socketpair");
    return ABNORMAL_RETURN;
  }

  child_pid = fork();  /* Creates child process */
  if (child_pid == 0) {
    dup2(sock[1], 0); /* Make 0 an alias of sock. */
    dup2(sock[1], 1); /* Same with 1 */
    close(sock[1]);   /* Close the extra fd. */
    
    /* Closes all files in chain, the child process should not
       have it's parents open files. */
    while (proc_file_chain) {
      close(proc_file_chain->fd);
      proc_file_chain = proc_file_chain->next;
    }

    execl("/bin/sh", "sh", "-c", command, (char *)0);
    perror("Could not exec command"); /* Where does this go? */
    _exit(EXIT_STATUS);
  }

  if (child_pid < 0) {
    close(sock[0]);
    close(sock[1]);
    return ABNORMAL_RETURN;
  }

  fpx->fd = sock[0];
  fpx->pid = child_pid;

  return child_pid; /* Isn't used anymore, it's in the fpx. */
}

int popenrw(const char *command)
{
  proc_file *fpx = new proc_file;

  if (fpx == NULL)
    return ABNORMAL_RETURN;

  /* Open the process and link into the chain. */
  if (proc_open(fpx, command) != ABNORMAL_RETURN) {
    fpx->next = proc_file_chain;
    proc_file_chain = fpx;
    return fpx->fd; /* The read/write fd to the process. */
  }

  delete fpx;
  return ABNORMAL_RETURN;
}

int pcloserw(int fd)
{
  int wstatus;
  proc_file **ptr;
  int wait_pid;
  proc_file *cur = NULL;
  
  /* Locate and unlink from proc_file_chain. */
  for (ptr = &proc_file_chain; *ptr != NULL; ptr = &(*ptr)->next)
    if (fd == (*ptr)->fd) {
      cur = *ptr;
      *ptr = (*ptr)->next;
      break;
    }

  if (cur != NULL) {
    do {
      wait_pid = waitpid(cur->pid, &wstatus, 0);
    } while ((wait_pid == -1) && (errno == EINTR));
    destroySock(cur->fd);
  } else
    wait_pid = -1;

  delete cur;

  if (wait_pid == -1)
    return ABNORMAL_RETURN;

  return wstatus;
}


